{$A+,B-,C-,D-,E-,F-,G+,H+,I-,J-,K-,L-,M-,N+,O+,P+,Q-,R-,S-,T-,U-,V+,W-,X+,Y-,Z1}

{*
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 *                                                                           *
 * Author:       Jerry Claxton                                               *
 * Description:  Displays Disk/File Usage in Selected Folders.               *
 * Creation:     May 18th, 2002                                              *
 * Version:      1.00                                                        *
 * EMail:        gclaxton@ameritech.net                                      *
 *               jerryclaxton@digitalme.com                                  *
 *                                                                           *
 * Support:      Use the email gclaxton@ameritech.net                        *
 * Legal issues: Copyright (C) 1996-2002 by Jerry Claxton                    *
 *               210 Village Creek Dr, Lake in the Hills, Illinois, 60156    *
 *               <gclaxton@ameritech.net> <jerryclaxton@digitalme.com>       *
 *                                                                           *
 *               This software is provided 'as-is', without any express or   *
 *               implied warranty.  In no event will the author be held      *
 *               liable for any  damages arising from the use of this        *
 *               software.                                                   *
 *                                                                           *
 *               Permission is granted to anyone to use this software for    *
 *               any purpose, including commercial applications, and to      *
 *               alter it and redistribute it freely, subject to the         *
 *               following restrictions:                                     *
 *                                                                           *
 *               1. The origin of this software must not be misrepresented,  *
 *                  you must not claim that you wrote the original software. *
 *                  If you use this software in a product, an acknowledgment *
 *                  in the product documentation would be appreciated but is *
 *                  not required.                                            *
 *                                                                           *
 *               2. Altered source versions must be plainly marked as such,  *
 *                  and must not be misrepresented as being the original     *
 *                  software.                                                *
 *                                                                           *
 *               3. This notice may not be removed or altered from any       *
 *                  source distribution.                                     *
 *                                                                           *
 *               4. You must register this software by sending an email to   *
 *                  the author. Any comment you like to say.                 *
 *                                                                           *
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 *                                                                           *
 * History:                                                                  *
 * May 20, 2002  First Release.                                              *
 *                                                                           *
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 *}

unit MainUnit;

interface

uses
  Windows, Messages, Dialogs, Classes, Graphics, Controls, Forms, StdCtrls,
  ComCtrls, ExtCtrls, ImgList, ScanThrd;

type
  TMainForm = class(TForm)
    Panel1: TPanel;
    Panel5: TPanel;
    Panel3: TPanel;
    ListView1: TListView;
    ListView2: TListView;
    Panel4: TPanel;
    Label1: TLabel;
    ImageList: TImageList;
    Panel2: TPanel;
    btnBegin: TButton;
    btnStop: TButton;
    btnExport: TButton;
    btnClose: TButton;
    Bevel3: TBevel;
    Image1: TImage;
    SaveDialog: TSaveDialog;
    Label2: TLabel;
    Label3: TLabel;
    edFolder: TEdit;
    btnBrowse: TButton;
    procedure btnBeginClick(Sender: TObject);
    procedure btnStopClick(Sender: TObject);
    procedure btnCloseClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure ListView1DblClick(Sender: TObject);
    procedure ListView1ColumnClick(Sender: TObject; Column: TListColumn);
    procedure ListView1Compare(Sender: TObject; Item1, Item2: TListItem;
      Data: Integer; var Compare: Integer);
    procedure btnExportClick(Sender: TObject);
    procedure btnBrowseClick(Sender: TObject);
    procedure FormCloseQuery(Sender: TObject; var CanClose: Boolean);
    procedure ListView1MouseDown(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
  private
    { Private declarations }
    ActiveThread: TMyThread;
    procedure ThreadDone(Sender: TObject);
    procedure ThreadProccess(Folder: string; TotalSize: Int64);
    procedure ThreadProgress(Folder: string);
  public
    { Public declarations }
    AllFoldersSize: Int64;
    SortColumn: Integer;
    SortForward: Boolean;
    procedure UpdateButtons;
    procedure UpdateHeader(Index: Integer; BitmapOn: Boolean);
  end;

var
  MainForm: TMainForm;

implementation

uses
  SysUtils, CommCtrl, ScopUnit, ItemProp;

{$R *.DFM}
procedure TMainForm.FormCreate(Sender: TObject);
begin
  ListView1.DoubleBuffered := True;
  ListView2.DoubleBuffered := True;
end;

procedure TMainForm.FormCloseQuery(Sender: TObject; var CanClose: Boolean);
begin
  if (ActiveThread <> nil) and Assigned(ActiveThread) then
  begin
    ActiveThread.Terminate;
    ActiveThread.WaitFor;
  end;
end;

procedure TMainForm.btnCloseClick(Sender: TObject);
begin
  Close;
end;

procedure TMainForm.ThreadProgress(Folder: string);
begin
  with ListView1.Items.Add do
  begin
    Caption := 'Scanning... ' + LowerCase(Folder);
    ImageIndex := 2;

    SubItems.Add('');
    SubItems.Add('');
  end;
  ListView1.Items[ListView1.Items.Count - 1].Focused := True;
  HighlightFocusedItem(ListView1);
end;

procedure TMainForm.ThreadProccess(Folder: string; TotalSize: Int64);
begin
  with ListView1.Items[ListView1.Items.Count - 1] do
  begin
    Caption := LowerCase(Folder);
    ImageIndex := 1;

    SubItems[0] := BytesToStr(TotalSize);
    SubItems[1] := FormatFloat('#0,', TotalSize);
  end;
  AllFoldersSize := AllFoldersSize + TotalSize;

  with ListView2.Items[0] do
  begin
    SubItems[0] := BytesToStr(AllFoldersSize);
    SubItems[1] := FormatFloat('#0,', AllFoldersSize);
  end;
end;

procedure TMainForm.ThreadDone(Sender: TObject);
begin
  if (ActiveThread <> nil) then ActiveThread := nil;

  with ListView1 do
  begin
    if Items.Count > 0 then
    begin
      SetFocus;
      Items[0].Focused := True;
    end;
    HighlightFocusedItem(ListView1);
  end;
  UpdateButtons;
end;

procedure TMainForm.btnBeginClick(Sender: TObject);
begin
  if (ActiveThread <> nil) and Assigned(ActiveThread) then Exit
  else begin
    UpdateButtons;

    with ListView1.Items.Add do
    begin
      Caption := '<root files>';

      AllFoldersSize := RootFileSize(edFolder.Text);
      SubItems.Add(BytesToStr(AllFoldersSize));
      SubItems.Add(FormatFloat('#0,', AllFoldersSize));

      with ListView2.Items.Add do
      begin
        Caption := 'Totals';
        SubItems.Add(BytesToStr(AllFoldersSize));
        SubItems.Add(FormatFloat('#0,', AllFoldersSize));
      end;
    end;

    ActiveThread := TScanThread.Create(AddBackSlash(edFolder.Text));
    with ActiveThread do
    begin
      OnProccess := ThreadProccess;
      OnProgress := ThreadProgress;
      OnTerminate := ThreadDone;
      Resume;
    end;
  end;
end;

procedure TMainForm.btnStopClick(Sender: TObject);
begin
  if (ActiveThread <> nil) and Assigned(ActiveThread) then
    ActiveThread.Terminate;
end;

procedure TMainForm.ListView1DblClick(Sender: TObject);
begin
  with ListView1 do
    if (Selected <> nil) and (SelCount = 1) then
    begin
      if Pos('<root files>', Items[Selected.Index].Caption) = 0 then
      begin
        edFolder.Text := Items[Selected.Index].Caption;
        btnBegin.Click;
      end;
    end;
end;

procedure TMainForm.ListView1ColumnClick(Sender: TObject; Column: TListColumn);
var
  Required_Column: integer;
begin
  if ((ActiveThread <> nil) and Assigned(ActiveThread)) or
    (ListView1.Items.Count = 0) then Exit;

  UpdateHeader(SortColumn, False);
  Required_Column := Column.Index;
  if Required_Column = SortColumn then
    SortForward := not SortForward
  else begin
    SortColumn := Required_Column;
    SortForward := False;
  end;
  ListView1.SortType := stBoth;
  ListView1.SortType := stNone;

  UpdateHeader(SortColumn, True);
  ListView1.Repaint;
end;

procedure TMainForm.ListView1Compare(Sender: TObject; Item1,
  Item2: TListItem; Data: Integer; var Compare: Integer);
var
  Result: Integer;
  size1, size2: Int64;
  Caption1, Caption2: string;
begin
  if ((ActiveThread <> nil) and Assigned(ActiveThread)) or
    (ListView1.Items.Count = 0) then Exit;
  Result := 0;

  if SortColumn = 0 then
  begin
    Caption1 := AnsiUpperCase(Item1.Caption);
    Caption2 := AnsiUpperCase(Item2.Caption);
    if Caption1 > Caption2 then
      Result := 1
    else
      if Caption1 < Caption2 then Result := -1;
  end

  else if (SortColumn = 1) or (SortColumn = 2) then
  begin
    Size1 := StrToInt64(DeleteComma(Item1.SubItems.Strings[1]));
    Size2 := StrToInt64(DeleteComma(Item2.SubItems.Strings[1]));

    if Size1 > Size2 then
      Result := 1
    else
      if Size1 < Size2 then Result := -1;
  end;

  if SortForward then
    Compare := -Result
  else Compare := Result;
end;

procedure TMainForm.UpdateHeader(Index: Integer; BitmapOn: Boolean);
var
  LVC: LV_COLUMN;
begin
  FillChar(LVC, SizeOf(LVC), 0);
  LVC.mask := LVCF_FMT;
  ListView1.Perform(LVM_GETCOLUMN, Index, Integer(@LVC));

  if not BitmapOn then
    LVC.fmt := LVC.fmt and not LVCFMT_IMAGE
  else begin
    LVC.mask := LVCF_FMT or LVCF_IMAGE;
    LVC.fmt := LVC.fmt or LVCFMT_IMAGE or LVCFMT_BITMAP_ON_RIGHT;
    if SortForward then
      LVC.iImage := 4
    else
      LVC.iImage := 3;
  end;
  ListView1.Perform(LVM_SETCOLUMN, Index, Integer(@LVC));
end;

procedure TMainForm.btnExportClick(Sender: TObject);
var
  TF: TextFile;
  N: Integer;
begin
  SaveDialog.Filter := 'Text files (*.txt)|*.TXT';
  SaveDialog.FileName := 'Folder Space.txt';
  SaveDialog.InitialDir := ExtractFilePath(Application.ExeName);

  if SaveDialog.Execute then
  begin
    try
      AssignFile(TF, SaveDialog.FileName);
      Rewrite(TF);

      WriteLn(TF, Format('%-68s  %12s  %15s', ['Folder Name', 'Folder Size', 'Size (bytes)']));
      WriteLn(TF, Format('%-68s  %12s  %15s', ['--------------------------------------------------------------------', '------------', '---------------']));
      for N := 0 to ListView1.Items.Count - 1 do
      begin
        with ListView1.Items[N] do
        begin
          WriteLn(TF, Format('%-68s  %12s  %15s', [Copy(Caption, 0, 68), SubItems[0], SubItems[1]]));
        end;
      end;
      WriteLn(TF, Format('%-68s  %12s  %15s', ['--------------------------------------------------------------------', '------------', '---------------']));
      with ListView2.Items[0] do
        WriteLn(TF, Format('%-68s  %12s  %15s', [Copy(Caption, 0, 68), SubItems[0], SubItems[1]]));
    finally
      CloseFile(TF);
    end;
    WinExec32('notepad.exe ' + SaveDialog.FileName, SW_SHOWNOACTIVATE);
  end;
end;

procedure TMainForm.btnBrowseClick(Sender: TObject);
var
  OutFolder: string;
begin
  SelectDirectory((Sender as TButton).Parent.Handle, 'Folder to Scan',
    edFolder.Text, '', False, OutFolder);

  if OutFolder <> '' then edFolder.Text := OutFolder;
  edFolder.SetFocus;
end;

procedure TMainForm.UpdateButtons;
begin
  AllFoldersSize := 0;

  btnBegin.Enabled := not btnBegin.Enabled;
  if not btnStop.Enabled then
  begin
    with ListView1.Items do if (ListView1.Items.Count > 0) then Clear;
    with ListView2.Items do if (ListView2.Items.Count > 0) then Clear;
  end;
  btnStop.Enabled := not btnStop.Enabled;
  btnBrowse.Enabled := not btnBrowse.Enabled;
  btnExport.Enabled := not btnExport.Enabled and (ListView1.Items.Count > 0);
  if btnStop.Enabled and not btnBegin.Enabled then btnStop.SetFocus;

  UpdateHeader(SortColumn, False);
end;

procedure TMainForm.ListView1MouseDown(Sender: TObject;
  Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
var
  Point: TPoint;
  B: Boolean;
begin
  if (ActiveThread <> nil) and Assigned(ActiveThread) then Exit;

  with ListView1 do
    if (Selected <> nil) and (SelCount = 1) then
    begin
      if Pos('<root files>', Items[Selected.Index].Caption) <> 0 then Exit;

      if Button = mbRight then
      begin
        B := False;
        GetCursorPos(Point);
        DisplayContextMenu(Items[Selected.Index].Caption,
          Handle, Point, False, B);
      end;
    end;
end;

end.

