{$A+,B-,C-,D-,E-,F-,G+,H+,I-,J-,K-,L-,M-,N+,O+,P+,Q-,R-,S-,T-,U-,V+,W-,X+,Y-,Z1}

{*
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 *                                                                           *
 * Author:       Jerry Claxton                                               *
 * Description:  Displays Disk/File Usage in Selected Folders.               *
 * Creation:     May 18th, 2002                                              *
 * Version:      1.00                                                        *
 * EMail:        gclaxton@ameritech.net                                      *
 *               jerryclaxton@digitalme.com                                  *
 *                                                                           *
 * Support:      Use the email gclaxton@ameritech.net                        *
 * Legal issues: Copyright (C) 1996-2002 by Jerry Claxton                    *
 *               210 Village Creek Dr, Lake in the Hills, Illinois, 60156    *
 *               <gclaxton@ameritech.net> <jerryclaxton@digitalme.com>       *
 *                                                                           *
 *               This software is provided 'as-is', without any express or   *
 *               implied warranty.  In no event will the author be held      *
 *               liable for any  damages arising from the use of this        *
 *               software.                                                   *
 *                                                                           *
 *               Permission is granted to anyone to use this software for    *
 *               any purpose, including commercial applications, and to      *
 *               alter it and redistribute it freely, subject to the         *
 *               following restrictions:                                     *
 *                                                                           *
 *               1. The origin of this software must not be misrepresented,  *
 *                  you must not claim that you wrote the original software. *
 *                  If you use this software in a product, an acknowledgment *
 *                  in the product documentation would be appreciated but is *
 *                  not required.                                            *
 *                                                                           *
 *               2. Altered source versions must be plainly marked as such,  *
 *                  and must not be misrepresented as being the original     *
 *                  software.                                                *
 *                                                                           *
 *               3. This notice may not be removed or altered from any       *
 *                  source distribution.                                     *
 *                                                                           *
 *               4. You must register this software by sending an email to   *
 *                  the author. Any comment you like to say.                 *
 *                                                                           *
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 *                                                                           *
 * History:                                                                  *
 * May 20, 2002  First Release.                                              *
 *                                                                           *
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 *}

unit ScopUnit;

interface

uses
  Windows, Forms, SysUtils, ComCtrls, Shlobj, ActiveX;

procedure HighlightFocusedItem(ListView: TListView);

function BytesToStr(const i64Size: Int64): string;
function DeleteComma(Path: string): string;
function AddBackSlash(const S: string): string;
function WinExec32(FileName: string; Visibility: integer): boolean;
function BrowseCallbackProc(hwnd: HWND; uMsg: UINT; lParam, lpData: LPARAM):
  Integer; stdcall;
function SelectDirectory(hOwner: THandle; const Caption, InitialDir: string;
  const Root: WideString; ShowStatus: Boolean; out Directory: string): Boolean;
function RootFileSize(const Folder: string): Int64;

implementation

procedure HighlightFocusedItem(ListView: TListView);
var
  Item: TListItem;
begin
  with ListView do
  begin
    Item := ItemFocused;
    if (Item <> nil) then
    begin
      Selected := Item;
      Item.MakeVisible(True);
    end;
  end;
end;

function BytesToStr(const i64Size: Int64): string;
const
  i64GB = 1024 * 1024 * 1024;
  i64MB = 1024 * 1024;
  i64KB = 1024;
begin
  if i64Size div i64GB > 0 then
    Result := Format('%.2f GB', [i64Size / i64GB])
  else if i64Size div i64MB > 0 then
    Result := Format('%.2f MB', [i64Size / i64MB])
  else if i64Size div i64KB > 0 then
    Result := Format('%.2f KB', [i64Size / i64KB])
  else
    Result := FormatFloat('#0,', i64Size);
end;

function AddBackSlash(const S: string): string;
begin
  Result := S;

  if Result[Length(Result)] <> '\' then
    Result := Result + '\';
end;

function DeleteComma(Path: string): string;
var
  I: integer;
  S: string;
begin
  for I := 1 to length(Path) do
    if (Path[I] <> ',') and (Path[I] <> ' ') then
      S := S + Format('%s', [Path[I]]);

  if S <> '' then Result := Trim(S)
  else Result := '0';
end;

function WinExec32(FileName: string; Visibility: integer): boolean;
var
  zAppName: array[0..512] of char;
  zCurDir: array[0..255] of char;
  WorkDir: string;
  StartupInfo: TStartupInfo;
  ProcessInfo: TProcessInformation;
begin
  StrPCopy(zAppName, FileName);
  GetDir(0, WorkDir);
  StrPCopy(zCurDir, WorkDir);
  FillChar(StartupInfo, Sizeof(StartupInfo), #0);
  StartupInfo.cb := Sizeof(StartupInfo);
  StartupInfo.dwFlags := STARTF_USESHOWWINDOW;
  StartupInfo.wShowWindow := Visibility;
  Result := CreateProcess(nil,
    zAppName,                                                                   { pointer to command line string }
    nil,                                                                        { pointer to process security attributes }
    nil,                                                                        { pointer to thread security attributes }
    False,                                                                      { handle inheritance flag }
    CREATE_NEW_CONSOLE or                                                       { creation flags }
    NORMAL_PRIORITY_CLASS,
    nil,                                                                        { pointer to new environment block }
    nil,                                                                        { pointer to current directory name }
    StartupInfo,                                                                { pointer to STARTUPINFO }
    ProcessInfo);                                                               { pointer to PROCESS_INF }

  CloseHandle(ProcessInfo.hProcess);
  CloseHandle(ProcessInfo.hThread);
end;

function BrowseCallbackProc(hwnd: HWND; uMsg: UINT; lParam, lpData: LPARAM):
  Integer; stdcall;
var
  Path: array[0..MAX_PATH] of Char;
begin
  case uMsg of
    BFFM_INITIALIZED:
      begin
        SendMessage(hwnd, BFFM_SETSELECTION, 1, lpData);
        SendMessage(hwnd, BFFM_SETSTATUSTEXT, 0, lpData);
      end;
    BFFM_SELCHANGED:
      begin
        if SHGetPathFromIDList(Pointer(lParam), Path) then
          SendMessage(hwnd, BFFM_SETSTATUSTEXT, 0, Integer(@Path));
      end;
  end;
  Result := 0;
end;

function SelectDirectory(hOwner: THandle; const Caption, InitialDir: string;
  const Root: WideString; ShowStatus: Boolean; out Directory: string): Boolean;
var
  BrowseInfo: TBrowseInfo;
  Buffer: PChar;
  RootItemIDList,
    ItemIDList: PItemIDList;
  ShellMalloc: IMalloc;
  IDesktopFolder: IShellFolder;
  Eaten, Flags: LongWord;
  Windows: Pointer;
  Path: string;
begin
  Result := False;
  Directory := '';
  Path := InitialDir;
  if (Length(Path) > 0) and (Path[Length(Path)] = '\') then
    Delete(Path, Length(Path), 1);

  FillChar(BrowseInfo, SizeOf(BrowseInfo), 0);
  if (ShGetMalloc(ShellMalloc) = S_OK) and (ShellMalloc <> nil) then
  begin
    Buffer := ShellMalloc.Alloc(MAX_PATH);
    try
      SHGetDesktopFolder(IDesktopFolder);
      IDesktopFolder.ParseDisplayName(hOwner, nil, PWideChar(Root), Eaten,
        RootItemIDList, Flags);
      with BrowseInfo do
      begin
        hwndOwner := hOwner;
        pidlRoot := RootItemIDList;
        pszDisplayName := Buffer;
        lpszTitle := PChar(Caption);
        ulFlags := BIF_RETURNONLYFSDIRS;
        if ShowStatus then
          ulFlags := ulFlags or BIF_STATUSTEXT;
        lParam := Integer(PChar(Path));
        lpfn := BrowseCallbackProc;
        iImage := 0;
      end;

      // Make the browser dialog modal.
      Windows := DisableTaskWindows(hOwner);
      try
        ItemIDList := ShBrowseForFolder(BrowseInfo);
      finally
        EnableTaskWindows(Windows);
      end;

      Result := ItemIDList <> nil;
      if Result then
      begin
        ShGetPathFromIDList(ItemIDList, Buffer);
        ShellMalloc.Free(ItemIDList);
        Directory := Buffer;
      end;
    finally
      ShellMalloc.Free(Buffer);
    end;
  end;
end;

function RootFileSize(const Folder: string): Int64;
var
  H: THandle;
  Data: TWin32FindData;
  fSize: Int64;
begin
  Result := 0;

  H := Windows.FindFirstFile(PChar(AddBackSlash(Folder) + '*.*'), Data);
  if H <> INVALID_HANDLE_VALUE then
    repeat
      if Data.dwFileAttributes and FILE_ATTRIBUTE_DIRECTORY = 0 then
      begin
        fSize := Data.nFileSizeHigh;
        fSize := fSize shl 32;
        Inc(Result, fSize + Data.nFileSizeLow);
      end;
    until not Windows.FindNextFile(H, Data);
  Windows.FindClose(H);
end;

end.

